import { ErrorNumberSnils } from "./exceptions/ErrorNumberSnils";
import { ISnils } from "./interfaces/ISnils";


export class SnilsBase implements ISnils {

    private _numberSnils: string | null = null;
    private _checksum: number | null = null;

    private splitSnils(snils: string): void {
        const splitSnils: Array<string> = snils.split(' ');
        this._numberSnils = splitSnils[0];
        this._checksum = +splitSnils[1];
    }

    private deleteSpecialSymbol(): string {
        if (this._numberSnils) {
            const numberSnils = this._numberSnils.replace(/\-/g, '');
            return numberSnils;
        }
        throw new ErrorNumberSnils();
    };
    private isRepeatCharacters(): boolean {
        const reg = new RegExp(/([0-9])\1{2,}/g);
        return reg.test(this.deleteSpecialSymbol());
    };

    private calculatingNumberSnilsSum(): number {
        let checksum: number = -1;
        const numberSnils: string = this.deleteSpecialSymbol();
        let countNumberSnils: number = numberSnils.length;
        if (countNumberSnils === 9 && !this.isRepeatCharacters()) {
            checksum = 0;
            let numberPosition: number = 0;
            for (let i = countNumberSnils; i !== 0; i--) {
                checksum += parseInt(numberSnils[numberPosition]) * i;
                numberPosition++;
            }
        }
        return checksum;
    };

    private calculatingChecksum(): number {
        const sumNumberSnils: number = this.calculatingNumberSnilsSum();
        let checksum: number = -1;
        if (sumNumberSnils !== -1) {
            if (sumNumberSnils === 100 || sumNumberSnils === 101) {
                return 0;
            }
            if (sumNumberSnils < 100) {
                return sumNumberSnils;
            }

            checksum = sumNumberSnils % 101;
        }
        return checksum;

    };
    public isValidationSnils(snils: string): boolean {
        try {
            this.splitSnils(snils);
            const checksum: number = this.calculatingChecksum();
            if (checksum !== -1 && checksum == this._checksum) {
                return true;
            }
        } catch (err) {
            console.error(err);
        }
        return false;
    };
}